#!/usr/bin/env python
import itertools
import random
import sys
import traceback
from pathlib import Path

import altair as alt
import pandas as pd
import plotly as plotly
import plotly.express as px
import plotly.graph_objects as go
import streamlit as st
from pandas.api.types import is_numeric_dtype
from plotly.validators.scatter.marker import SymbolValidator
from streamlit import cli as stcli

import scml_vis.compiler as compiler
from scml_vis.compiler import VISDATA_FOLDER
from scml_vis.utils import (
    add_selector,
    add_stats_display,
    add_stats_selector,
    load_data,
    plot_network,
    score_distribution,
    score_factors,
)

__all__ = ["main"]


MARKERS = SymbolValidator().values[2::3]
MARKERS = [_ for _ in MARKERS if not any(_.startswith(x) for x in ("star", "circle", "square"))]
random.shuffle(MARKERS)
MARKERS = ["circle", "square"] + MARKERS

DB_FOLDER = Path.home() / "negmas" / "runsdb"
DB_NAME = "rundb.csv"
BASE_FOLDERS = [
    Path.home() / "negmas" / "logs" / "scml" / "scml2020",
    Path.home() / "negmas" / "logs" / "scml" / "scml2020oneshot",
    Path.home() / "negmas" / "logs" / "scml" / "scml2021oneshot",
    Path.home() / "negmas" / "logs" / "scml" / "scml2021",
    Path.home() / "negmas" / "logs" / "tournaments",
]


def main(folder: Path):
    st.set_page_config(layout="wide")
    if folder is None:
        options = dict(none="none")
        if (DB_FOLDER / DB_NAME).exists():
            data = pd.read_csv(DB_FOLDER / DB_NAME, index_col=None, header=None)
            data: pd.DataFrame
            data = data.iloc[::-1]
            data.columns = ["name", "type", "path"]
            for _, x in data.iterrows():
                options[x["path"]] = f"{x['type'][0]}:{x['name']}"
        add_base = st.sidebar.checkbox("Add Default paths", True)
        if add_base:
            for base in BASE_FOLDERS:
                type_ = base.name == "tournaments"
                for child in base.glob("*"):
                    if not child.is_dir() or not compiler.has_logs(child):
                        continue
                    options[child] = f"{'t' if type_ else 'w'}:{child.name}"

        folder = st.sidebar.selectbox("Select a run", list(options.keys()), format_func=lambda x: options[x])
    if not folder or (isinstance(folder, str) and folder == "none"):
        st.text(
            "Cannot find any folders with logs.\nTry looking in default paths by checking 'Add Default paths' \nin the side bar or start the app with a folder containing log data using -f"
        )
        return
    folder = Path(folder)
    if folder.name != VISDATA_FOLDER:
        folder = folder / VISDATA_FOLDER
    if not folder.exists():
        try:
            do_compile = st.sidebar.button("Compile visualization data?")
            if do_compile:
                try:
                    compiler.main(folder.parent, max_worlds=None)
                except Exception as e:
                    st.write(f"*Failed to compile visualization data for {folder}*\n### Exception:\n{str(e)}")
                    st.write(f"\n### Traceback:\n```\n{traceback.format_exc()}```")
            else:
                st.text("Either press 'Compile visualization data' to view logs of this folder or choose another one.")
                return
        except:
            st.write(f"Folder {folder} contains no logs to use")
        # folder = folder / VISDATA_FOLDER
        # if not folder.exists():
        #     st.write(
        #         f"## SCML Visualizer\nError: No {VISDATA_FOLDER} folder found with visualization data at {str(folder)}"
        #     )
        #     return
    if folder.name != VISDATA_FOLDER:
        folder = folder / VISDATA_FOLDER
    if not folder.exists():
        st.write("Cannot find visualiation data")
        return

    st.write(f"## SCML Visualizer\n{str(folder.parent)}")

    st.sidebar.markdown("## Data Selection")
    tournaments = load_data(folder, "tournaments")
    tournament_expander = st.sidebar.beta_expander("Tournament Selection")
    with tournament_expander:
        selected_tournaments, _, _ = add_selector(
            st,
            "",
            tournaments.name.unique(),
            key="tournaments",
            none=False,
            default="one",
        )
    worlds = None
    configs = load_data(folder, "configs")
    if configs is None:
        worlds = load_data(folder, "worlds")
        config_names = worlds.loc[:, "name"].str.split("_").str[0].unique()
        configs = pd.DataFrame(data=config_names, columns=["id"])
    config_expander = st.sidebar.beta_expander("Config Selection")
    with config_expander:
        selected_configs, _, _ = add_selector(
            st,
            "",
            configs["id"].unique(),
            key="configs",
            none=False,
            default="all",
        )

    if worlds is None:
        worlds = load_data(folder, "worlds")
    if "config" not in worlds.columns:
        worlds["config"] = worlds.loc[:, "name"].str.split("_").str[0]
    worlds = worlds.loc[worlds.tournament.isin(selected_tournaments) & worlds.config.isin(selected_configs), :]
    world_expander = st.sidebar.beta_expander("World Selection")
    with world_expander:
        selected_worlds, _, _ = add_selector(st, "", worlds.name, key="worlds", none=False, default="all")
    worlds = worlds.loc[(worlds.name.isin(selected_worlds)), :]

    agents = load_data(folder, "agents")
    type_expander = st.sidebar.beta_expander("Type Selection")
    with type_expander:
        selected_types, _, _ = add_selector(st, "", agents.type.unique(), key="types", none=False, default="all")
    agents = agents.loc[(agents.type.isin(selected_types)), :]

    agent_expander = st.sidebar.beta_expander("Agent Selection")
    with agent_expander:
        selected_agents, _, _ = add_selector(st, "", agents.name.unique(), key="agents", none=False, default="all")

    products = load_data(folder, "product_stats")
    product_expander = st.sidebar.beta_expander("Product Selection")
    with product_expander:
        selected_products, _, _ = add_selector(
            st, "", products["product"].unique(), key="products", none=False, default="all"
        )

    agents = agents.loc[(agents.type.isin(selected_types)), :]

    nsteps = worlds.loc[worlds.name.isin(selected_worlds), "n_steps"].max()
    nsteps = int(nsteps)
    selected_steps = st.sidebar.slider("Steps", 0, nsteps, (0, nsteps))
    selected_times = st.sidebar.slider("Relative Times", 0.0, 1.0, (0.0, 1.0))

    st.sidebar.markdown("## Figure Selection")
    # ts_figs = st.sidebar.beta_expander("Time Series")
    # net_figs = st.sidebar.beta_expander("Networks")
    # tbl_figs = st.sidebar.beta_expander("Tables")
    # other_figs = st.sidebar.beta_expander("Others")
    #     if len(selected_worlds) == 1:
    #         fig_type = st.sidebar.selectbox(label="", options=["Time-series", "Networks", "Tables", "Others"], index=1)
    #     else:
    #         fig_type = st.sidebar.selectbox(label="", options=["Time-series", "Tables", "Others"], index=1)
    #
    #     if fig_type == "Time-series":
    #         runner = display_time_series
    #     elif fig_type == "Networks":
    #         runner = display_networks
    #     elif fig_type == "Tables":
    #         runner = display_tables
    #     elif fig_type == "Others":
    #         runner = display_others
    #     else:
    #         st.text("Please choose what type of figures are you interested in")
    #         return
    products_summary = (
        products.loc[:, [_ for _ in products.columns if _ not in ("step", "relative_time")]]
        .groupby(["tournament", "world", "product"])
        .agg(["min", "max", "mean", "std"])
    )
    products_summary.columns = [f"{a}_{b}" for a, b in products_summary.columns]
    products_summary = products_summary.reset_index()
    data = dict(t=tournaments, w=worlds, a=agents, p=products_summary)

    def filter(x, agent_field_sets):
        if x is None:
            return x
        x = x.loc[(x.world.isin(selected_worlds)), :]
        indx = None
        for fields in agent_field_sets:
            if not fields:
                continue
            indx = x[fields[0]].isin(selected_agents)
            for f in fields[1:]:
                indx = (indx) | (x[f].isin(selected_agents))
        if indx is None:
            return x
        return x.loc[indx, :]

    data["con"] = load_data(folder, "configs")
    data["a"] = load_data(folder, "agents")
    data["t"] = load_data(folder, "types")
    data["c"] = filter(load_data(folder, "contracts"), [["buyer", "seller"]])
    data["n"] = filter(load_data(folder, "negotiations"), [["buyer", "seller"]])
    data["o"] = filter(load_data(folder, "offers"), [["sender", "receiver"]])
    for runner, section_name in [
        (display_networks, "Networks"),
        (display_others, "Overview"),
        (display_tables, "Tables"),
        (display_time_series, "Time Series"),
    ]:
        if section_name != "Time Series":
            expander = st.sidebar.beta_expander(section_name, section_name == "Networks")
            do_expand = expander.checkbox(f"Show {section_name}", section_name == "Networks")
        else:
            expander = st.sidebar
            do_expand = st.sidebar.checkbox(section_name, True)
        if do_expand:
            runner(
                folder,
                selected_worlds,
                selected_products,
                selected_agents,
                selected_types,
                selected_steps,
                selected_times,
                data,
                parent=expander,
            )
            # st.sidebar.markdown("""---""")


def filter_by_time(x, cols, selected_steps, selected_times):
    indx = None
    for k in cols:
        step_col, time_col = f"{k}step", f"{k}relative_time"
        i = (x[step_col] >= selected_steps[0]) & (x[step_col] <= selected_steps[1])
        i &= (x[time_col] >= selected_times[0]) & (x[time_col] <= selected_times[1])
        if indx is None:
            indx = i
        else:
            indx |= i
    if indx is not None:
        return x.loc[indx, :]
    return x


def show_a_world(
    world,
    selected_steps,
    selected_times,
    data,
    parent,
    weight_field,
    edge_weights,
    edge_colors,
    node_weight,
    condition_field,
    x,
    src,
    gallery,
):
    nodes = data["a"].loc[data["a"].world == world, :]
    nodes["score*cost"] = nodes["final_score"] * nodes["cost"]
    fields = [_ for _ in nodes.columns]
    nodes = nodes.to_dict("records")
    added = -data["a"].input_product.min()
    nlevels = data["a"].input_product.max() + 1 + added

    level_max = [0] * (nlevels)
    dx, dy = 10, 10
    for node in nodes:
        l = node["input_product"] + added
        node["pos"] = ((l + 1) * dx, level_max[l] * dy)
        level_max[l] += 1

    nodes = {n["name"]: n for n in nodes}
    seller_dict = dict(zip(fields, itertools.repeat(float("nan"))))
    buyer_dict = dict(zip(fields, itertools.repeat(float("nan"))))
    nodes["SELLER"] = {**seller_dict, **dict(pos=(0, dy * (level_max[0] // 2)), name="Seller", type="System")}
    nodes["BUYER"] = {
        **buyer_dict,
        **dict(pos=((nlevels + 1) * dx, dy * (level_max[-1] // 2)), name="Buyer", type="System"),
    }
    edges, weights = [], []
    weight_field_name = "quantity" if weight_field == "count" else weight_field
    time_cols = (
        [condition_field + "_step", condition_field + "_relative_time"]
        if condition_field != "step"
        else ["step", "relative_time"]
    )
    x = x.loc[x.world == world, [weight_field_name, "seller", "buyer"] + time_cols]
    x = filter_by_time(x, [condition_field + "_" if condition_field != "step" else ""], selected_steps, selected_times)
    x.drop(time_cols, axis=1, inplace=True)
    if weight_field == "unit_price":
        x = x.groupby(["seller", "buyer"]).mean().reset_index()
        x["unit_price"].fillna(0.0, inplace=True)
    elif weight_field == "count":
        x = x.groupby(["seller", "buyer"]).count().reset_index()
        x.rename(columns=dict(quantity="count"), inplace=True)
    else:
        x = x.groupby(["seller", "buyer"]).sum().reset_index()
    for _, d in x.iterrows():
        edges.append((d["seller"], d["buyer"], d[weight_field]))
    parent.plotly_chart(
        plot_network(
            fields, nodes, edges=edges, node_weights=node_weight, edge_colors=edge_colors, edge_weights=edge_weights
        )
    )
    if gallery:
        return

    col1, col2 = parent.beta_columns(2)
    mydata = data[src]
    myselected = mydata.loc[(mydata.world == world), :]
    myselected = filter_by_time(
        myselected, [condition_field + "_" if condition_field != "step" else ""], selected_steps, selected_times
    )
    seller = col1.selectbox("Seller", [""] + sorted(x["seller"].unique()), key=f"seller-{world}")
    buyer = col2.selectbox("Buyer", [""] + sorted(x["buyer"].unique()), key=f"seller-{world}")
    if seller:
        myselected = myselected.loc[(myselected.seller == seller), :]
    if buyer:
        myselected = myselected.loc[(myselected.buyer == buyer), :]
    myselected = myselected.reset_index()
    options = myselected
    if src == "n":
        col1, col2 = parent.beta_columns(2)
        broken = col1.checkbox("Broken", False, key=f"broken-{world}")
        timedout = col2.checkbox("Timedout", False, key=f"timedout-{world}")
        if not broken:
            options = options.loc[~options.broken, :]
        if not timedout:
            options = options.loc[~options.timedout, :]

    # options = options.loc[(options["seller"]==seller) & (options["buyer"]==buyer) & (options.world == world) & (options[f"{condition_field}_step"]<= selected_steps[1]) & (options[f"{condition_field}_step"]>= selected_steps[0]) , :]

    if src == "c":
        displayed_cols = (
            [
                "id",
                "delivery_step",
                "quantity",
                "unit_price",
                "total_price",
                "n_neg_steps",
                "concluded_step",
                "signed_step",
                "executed_step",
                "negotiation",
            ]
            + (["buyer"] if not buyer else [])
            + (["seller"] if not seller else [])
        )
    elif src == "n":
        displayed_cols = ["id", "delivery_step", "quantity", "unit_price", "timedout", "broken", "step", "rounds"]
    else:
        return
    parent.dataframe(
        myselected.loc[:, displayed_cols].sort_values(
            ["signed_step", "delivery_step"] if src == "c" else ["step", "delivery_step"]
        )
    )
    contract = None

    options = filter_by_time(
        options, [condition_field + "_" if condition_field != "step" else ""], selected_steps, selected_times
    )
    if parent.checkbox("Ignore Exogenous", key=f"ignore-exogenous-{world}"):
        options = options.loc[(options["buyer"] != "BUYER") & (options["seller"] != "SELLER"), :]
    if src == "n":
        options = options.loc[:, "id"].values
        if len(options) < 1:
            return
        neg = parent.selectbox(label="Negotiation", options=options, key=f"negotiationselect-{world}")
    elif src == "c":
        options = options.loc[:, "id"].values
        if len(options) < 1:
            return
        elif len(options) == 1:
            contract = options[0]
        else:
            contract = parent.selectbox(label="Contract", options=options, key=f"contractselect-{world}")
        neg = myselected.loc[myselected["id"] == contract, "negotiation"]
        if len(neg) > 0:
            neg = neg.values[0]
        else:
            neg = None
    else:
        return
    if contract is not None:
        parent.write(data["c"].loc[data["c"]["id"] == contract, :])
    if not neg or data["n"] is None or len(data["n"]) == 0:
        return
    neg_info = data["n"].loc[data["n"]["id"] == neg]
    offers = data["o"]
    offers = offers.loc[offers.negotiation == neg, :].sort_values(["round", "sender"])
    # if len(offers) >= 2:
    #     offers = offers.loc[offers["sender"].shift(1) != offers["sender"],:]
    offers.index = range(len(offers))

    parent.write(neg_info)
    if len(neg_info) < 1:
        return
    neg_info = neg_info.to_dict("records")[0]
    if not neg_info["broken"] and not neg_info["timedout"]:
        agreement = dict(
            quantity=neg_info["quantity"],
            delivery_step=neg_info["delivery_step"],
            unit_price=neg_info["unit_price"],
            total_price=neg_info["unit_price"] * neg_info["quantity"],
        )
    else:
        agreement = None
    parent.markdown(f"**Agreement**: {agreement}")

    trange = (neg_info["min_delivery_step"], neg_info["max_delivery_step"])
    c1, c2 = parent.beta_columns(2)
    if trange[1] > trange[0]:
        is_3d = c2.checkbox("3D Graph", key=f"threed-{world}")
    else:
        is_3d = False
    use_ranges = c1.checkbox("Use issue ranges to set axes", True, key=f"useissueranges-{world}")
    qrange = (neg_info["min_quantity"] - 1, neg_info["max_quantity"] + 1)
    urange = (neg_info["min_unit_price"] - 1, neg_info["max_unit_price"] + 1)
    if is_3d:
        fig = go.Figure()
        for i, sender in enumerate(offers["sender"].unique()):
            myoffers = offers.loc[offers["sender"] == sender, :]
            fig.add_trace(
                go.Scatter3d(
                    x=myoffers["quantity"],
                    y=myoffers["unit_price"],
                    z=myoffers["delivery_step"],
                    name=sender,
                    mode="lines+markers",
                    marker=dict(size=10),
                    marker_symbol=MARKERS[i],
                )
            )
        if agreement:
            fig.add_trace(
                go.Scatter3d(
                    x=[agreement["quantity"]],
                    y=[agreement["unit_price"]],
                    z=[agreement["delivery_step"]],
                    mode="markers",
                    marker=dict(size=20),
                    name="Agreement",
                    marker_symbol="diamond",
                )
            )
        fig.update_layout(xaxis_title="quantity", yaxis_title="unit_price")
    else:
        fig = go.Figure()
        for i, sender in enumerate(offers["sender"].unique()):
            myoffers = offers.loc[offers["sender"] == sender, :]
            fig.add_trace(
                go.Scatter(
                    x=myoffers["quantity"],
                    y=myoffers["unit_price"],
                    name=sender,
                    mode="lines+markers",
                    marker=dict(size=10),
                    marker_symbol=MARKERS[i],
                )
            )
        if agreement:
            fig.add_trace(
                go.Scatter(
                    x=[agreement["quantity"]],
                    y=[agreement["unit_price"]],
                    mode="markers",
                    marker=dict(size=20),
                    name="Agreement",
                    marker_symbol="star",
                )
            )
        fig.update_layout(xaxis_title="quantity", yaxis_title="unit_price")
        if use_ranges:
            fig.update_layout(xaxis_range=qrange, yaxis_range=urange)
    col1, col2 = parent.beta_columns(2)

    def fig_1d(y):
        fig = go.Figure()
        for i, sender in enumerate(offers["sender"].unique()):
            myoffers = offers.loc[offers["sender"] == sender, :]
            fig.add_trace(
                go.Scatter(
                    x=myoffers["round"],
                    y=myoffers[y],
                    name=sender,
                    mode="lines+markers",
                    marker=dict(size=15),
                    marker_symbol=MARKERS[i],
                )
            )
        if agreement:
            fig.add_trace(
                go.Scatter(
                    x=[offers["round"].max()],
                    y=[agreement[y]],
                    mode="markers",
                    marker=dict(size=20),
                    name="Agreement",
                    marker_symbol="star",
                )
            )
        fig.update_layout(xaxis_title="Round", yaxis_title=y)
        fig.update_layout(yaxis_range=urange if y == "unit_price" else qrange if y == "quantity" else trange)
        return fig

    col1.plotly_chart(fig_1d("quantity"))
    col1.plotly_chart(fig)
    col2.plotly_chart(fig_1d("unit_price"))
    if trange[1] > trange[0]:
        col2.plotly_chart(fig_1d("delivery_step"))

    parent.dataframe(offers)


WORLD_INDEX = 0


def display_networks(
    folder,
    selected_worlds,
    selected_products,
    selected_agents,
    selected_types,
    selected_steps,
    selected_times,
    data,
    parent=st.sidebar,
):
    global WORLD_INDEX
    max_worlds = parent.number_input("Max. Worlds", 1, None, 4)

    if len(selected_worlds) < 1:
        st.write("No worlds selected. Cannot show any networks")
        return
    if len(selected_worlds) > max_worlds:
        st.write(f"More than {max_worlds} world selected ({len(selected_worlds)}). Will show the first {max_worlds}")
        cols = st.beta_columns([1, 5, 1, 3])
        # prev = cols[0].button("<")
        # next = cols[2].button(">")
        # if prev:
        #     WORLD_INDEX = (WORLD_INDEX - max_worlds) % len(selected_worlds)
        # if next:
        #     WORLD_INDEX = (WORLD_INDEX + max_worlds) % len(selected_worlds)
        WORLD_INDEX = cols[1].slider("", 0, len(selected_worlds) - 1, WORLD_INDEX)
        randomize = cols[3].button("Randomize worlds")
        if randomize:
            random.shuffle(selected_worlds)
        selected_worlds = selected_worlds[WORLD_INDEX : WORLD_INDEX + max_worlds]
    what = parent.selectbox("Category", ["Contracts", "Negotiations"])
    if what == "Contracts":
        src = "c"
    elif what == "Negotiations":
        src = "n"
    else:
        src = "o"
    x = data[src]
    if x is None:
        st.markdown(f"**{what}** data is **not** available in the logs.")
        return
    gallery = parent.checkbox("Gallery Mode", len(selected_worlds) > 1)
    node_weight_options = sorted(
        [_ for _ in data["a"].columns if is_numeric_dtype(data["a"][_]) and _ not in ("id", "is_default")]
    )
    default_node_weight = node_weight_options.index("final_score")
    if default_node_weight is None:
        default_node_weight = 0
    with st.beta_expander("Networks Settings"):
        cols = st.beta_columns(5 + int(gallery))
        weight_field = cols[2].selectbox("Edge Weight", ["total_price", "unit_price", "quantity", "count"])
        node_weight = cols[3].selectbox("Node Weight", ["none"] + node_weight_options, default_node_weight + 1)
        per_step = cols[0].checkbox("Show one step only")
        edge_weights = cols[0].checkbox("Variable Edge Width", True)
        edge_colors = cols[0].checkbox("Variable Edge Colors", True)
        if per_step:
            selected_step = cols[1].number_input("Step", selected_steps[0], selected_steps[1], selected_steps[0])
            selected_steps = [selected_step] * 2
        x["total_price"] = x.quantity * x.unit_price
        options = [_[: -len("_step")] for _ in x.columns if _.endswith("_step")]
        if src != "c":
            options.append("step")
        condition_field = cols[4].selectbox("Condition", options, 0 if src != "n" else options.index("step"))
    if gallery:
        n_cols = cols[5].number_input("Columns", 1, 5, 2)
        cols = st.beta_columns(n_cols)
    else:
        n_cols, cols = 1, [st]

    for i, world in enumerate(selected_worlds):
        show_a_world(
            world,
            selected_steps=selected_steps,
            selected_times=selected_times,
            data=data,
            parent=cols[i % n_cols],
            weight_field=weight_field,
            edge_weights=edge_weights,
            edge_colors=edge_colors,
            node_weight=node_weight,
            condition_field=condition_field,
            x=x,
            src=src,
            gallery=gallery,
        )


def display_tables(
    folder,
    selected_worlds,
    selected_products,
    selected_agents,
    selected_types,
    selected_steps,
    selected_times,
    data,
    parent=st.sidebar,
):
    remove_single = parent.checkbox("Remove fields with a single value", True)

    def order_columns(x):
        cols = sorted(x.columns)
        for c in ["buyer_type", "seller_type", "delivery_step", "quantity", "unit_price", "total_price", "buyer", "seller", "name", "id"]:
            if c in cols:
                cols = [c] + [_ for _ in cols if _ != c]
        for c in ["world", "config", "group", "tournament"]:
            if c in cols:
                cols = [_ for _ in cols if _ != c] + [c]
        return x.loc[:, cols]

    def remove_singletons(x):
        selected = []
        for c in x.columns:
            if len(x[c].unique()) < 2:
                continue
            selected.append(c)
        return x.loc[:, selected]

    def show_table(x, must_choose=False):
        x = order_columns(x)
        if remove_single:
            x = remove_singletons(x)
        selected_cols = st.multiselect(label="Columns", options=x.columns)
        if selected_cols or must_choose:
            st.dataframe(x.loc[:, selected_cols])
        else:
            st.dataframe(x)

    def create_chart(df, type):
        if type == "Scatter":
            return alt.Chart(df).mark_point()
        if type == "Bar":
            return alt.Chart(df).mark_bar()
        if type == "Box":
            return alt.Chart(df).mark_boxplot()
        if type == "Line":
            return alt.Chart(df).mark_line()
        raise ValueError(f"Unknown marker type {type}")

    for lbl, k, has_step in (
        ("Tournaments", "t", False),
        ("Configs", "con", False),
        ("Worlds", "w", False),
        ("Products", "p", False),
        ("Agents", "a", False),
        ("Contracts", "c", True),
        ("Negotiations", "n", True),
        ("Offers", "o", True),
    ):

        if data[k] is None or not len(data[k]):
            continue
        if not parent.checkbox(label=lbl):
            continue
        if has_step:
            df = filter_by_time(
                data[k], ["signed_", "concluded_"] if k == "c" else [""], selected_steps, selected_times
            )
        else:
            df = data[k]
        if lbl == "Agents":
            if st.checkbox("Ignore Default Agents", True):
                df = df.loc[~df["is_default"], :]
        elif lbl == "Contracts":
            if st.checkbox("Ignore Exogenous Contracts", True):
                df = df.loc[df["n_neg_steps"] < 1, :]
        show_table(df)
        st.text(f"{len(df)} records found")
        cols = st.beta_columns(6)
        type_ = cols[0].selectbox("Chart", ["Scatter", "Line", "Bar", "Box"], 0)
        x = cols[1].selectbox("x", ["none"] + list(df.columns))
        y = m = c = s = "none"
        if x != "none":
            y = cols[2].selectbox("y", ["none"] + list(df.columns))
            if y != "none":
                m = cols[3].selectbox("Mark", ["none"] + list(df.columns))
                c = cols[4].selectbox("Color", ["none"] + list(df.columns))
                s = cols[5].selectbox("Size", ["none"] + list(df.columns))
                kwargs = dict(x=x, y=y)
                if m != "none": kwargs["shape"] = m
                if s != "none": kwargs["size"] = s
                if c != "none": kwargs["color"] = c
            else:
                kwargs = dict(x=x, y=alt.X(x, bin=True))
            chart = create_chart(df, type_ if y != "none" else "Bar").encode(**kwargs)
            st.altair_chart(chart, use_container_width=True)


def display_time_series(
    folder,
    selected_worlds,
    selected_products,
    selected_agents,
    selected_types,
    selected_steps,
    selected_times,
    data,
    parent=st.sidebar,
):
    settings = st.beta_expander("Time Series Settings")
    ncols = settings.number_input("N. Columns", min_value=1, max_value=6)
    xvar = settings.selectbox("x-variable", ["step", "relative_time"], int(len(selected_worlds) == 1))
    dynamic = settings.checkbox("Dynamic Figures", value=True)
    sectioned = settings.checkbox("Figure Sections", True)
    ci_level = settings.selectbox(options=[80, 90, 95], label="CI Level", index=2)
    world_stats, selected_world_stats, combine_world_stats, overlay_world_stats = add_stats_selector(
        folder,
        "world_stats",
        [[("world", selected_worlds), ("step", selected_steps), ("relative_time", selected_times)]],
        xvar=xvar,
        label="World Statistics",
        choices=lambda x: [
            _ for _ in x.columns if _ not in ("name", "world", "name", "tournament", "type", "step", "relative_time")
        ],
        default_selector="one",
    )

    product_stats, selected_product_stats, combine_product_stats, overlay_product_stats = add_stats_selector(
        folder,
        "product_stats",
        [[("product", selected_products), ("step", selected_steps), ("relative_time", selected_times)]],
        xvar=xvar,
        label="Product Statistics",
        choices=lambda x: [
            _
            for _ in x.columns
            if _ not in ("name", "world", "name", "tournament", "type", "step", "product", "relative_time")
        ],
        default_selector="some",
        default_choice=["trading_price"],
        combine=False,
    )

    type_stats, selected_type_stats, combine_type_stats, overlay_type_stats = add_stats_selector(
        folder,
        "agent_stats",
        [
            [
                ("world", selected_worlds),
                ("type", selected_types),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ]
        ],
        xvar=xvar,
        label="Type Statistics",
        choices=lambda x: [
            _ for _ in x.columns if _ not in ("name", "world", "name", "tournament", "type", "step", "relative_time")
        ],
        key="type",
        default_selector="some" if len(selected_worlds) != 1 else "none",
        default_choice=["score"] if len(selected_worlds) != 1 else None,
        combine=False,
    )

    agent_stats, selected_agent_stats, combine_agent_stats, overlay_agent_stats = add_stats_selector(
        folder,
        "agent_stats",
        [
            [
                ("world", selected_worlds),
                ("name", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ]
        ],
        xvar=xvar,
        label="Agent Statistics",
        choices=lambda x: [
            _ for _ in x.columns if _ not in ("name", "world", "name", "tournament", "type", "step", "relative_time")
        ],
        default_selector="some" if len(selected_worlds) == 1 else "none",
        default_choice=["score"] if len(selected_worlds) == 1 else None,
        combine=False,
    )

    (
        contract_stats_world,
        selected_contract_stats_world,
        combine_contract_stats_world,
        overlay_contract_stats_world,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (World)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="world",
    )

    (
        contract_stats_type,
        selected_contract_stats_type,
        combine_contract_stats_type,
        overlay_contract_stats_type,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Types)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="type",
    )
    (
        contract_stats_agent,
        selected_contract_stats_agent,
        combine_contract_stats_agent,
        overlay_contract_stats_agent,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Agents)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="name",
    )

    def aggregate_contract_stats(stats, ignored_cols):
        cols = [_ for _ in stats.columns if not any(_.endswith(x) for x in ["price", "quantity", "count"])]
        ignored_cols = [_ for _ in cols if _.startswith(ignored_cols)]
        cols = [_ for _ in cols if not _ in ignored_cols]
        allcols = [_ for _ in stats.columns if not _ in ignored_cols]
        # st.text(stats.columns)
        # st.text(allcols)
        # st.text(cols)
        # st.text(len(stats))
        stats = stats.loc[:, allcols].groupby(cols).sum()
        # st.text(len(stats))
        for c in stats.columns:
            if c.endswith("unit_price"):
                base = "_".join(c.split("_")[:-2])
                stats[c] = stats[f"{base}_total_price"] / stats[f"{base}_quantity"]
                stats[c] = stats[c].fillna(0)
        # st.text(len(stats))
        return stats.reset_index()

    (
        contract_stats_buyer_type,
        selected_contract_stats_buyer_type,
        combine_contract_stats_buyer_type,
        overlay_contract_stats_buyer_type,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            # [("world", selected_worlds), ("seller", selected_agents)],
        ],
        xvar=xvar,
        label="Contract Statistics (Buyer Types)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="buyer_type",
    )
    (
        contract_stats_seller_type,
        selected_contract_stats_seller_type,
        combine_contract_stats_seller_type,
        overlay_contract_stats_seller_type,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            # [("world", selected_worlds), ("buyer", selected_agents)],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Seller Types)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="seller_type",
    )
    (
        contract_stats_buyer,
        selected_contract_stats_buyer,
        combine_contract_stats_buyer,
        overlay_contract_stats_buyer,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            # [("world", selected_worlds), ("seller", selected_agents)],
        ],
        xvar=xvar,
        label="Contract Statistics (Buyer)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="buyer",
    )

    (
        contract_stats_seller,
        selected_contract_stats_seller,
        combine_contract_stats_seller,
        overlay_contract_stats_seller,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            # [("world", selected_worlds), ("buyer", selected_agents)],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Seller)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="seller",
    )

    contract_stats_buyer = aggregate_contract_stats(contract_stats_buyer, "seller")
    contract_stats_seller = aggregate_contract_stats(contract_stats_seller, "buyer")
    contract_stats_buyer_type = aggregate_contract_stats(contract_stats_buyer, "seller_type")
    contract_stats_seller_type = aggregate_contract_stats(contract_stats_seller, "buyer_type")

    contract_stats_agent["agent"] = contract_stats_agent["seller"] + "->" + contract_stats_agent["buyer"]
    contract_stats_agent["agent_type"] = contract_stats_agent["seller_type"] + "->" + contract_stats_agent["buyer_type"]
    contract_stats_type["agent"] = contract_stats_type["seller"] + "->" + contract_stats_type["buyer"]
    contract_stats_type["agent_type"] = contract_stats_type["seller_type"] + "->" + contract_stats_type["buyer_type"]

    (
        contract_stats_product,
        selected_contract_stats_product,
        combine_contract_stats_product,
        overlay_contract_stats_product,
    ) = add_stats_selector(
        folder,
        "contract_stats",
        [
            [
                ("world", selected_worlds),
                ("buyer", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
            [
                ("world", selected_worlds),
                ("seller", selected_agents),
                ("step", selected_steps),
                ("relative_time", selected_times),
            ],
        ],
        xvar=xvar,
        label="Contract Statistics (Product)",
        default_selector="none",
        choices=lambda x: [
            _ for _ in x.columns if _.endswith("quantity") or _.endswith("count") or _.endswith("price")
        ],
        key="product",
    )

    cols, start_col = add_stats_display(
        world_stats,
        selected_world_stats,
        combine_world_stats,
        overlay_world_stats,
        ncols=ncols,
        xvar=xvar,
        hue="world",
        title="World Figures",
        sectioned=sectioned,
        cols=None,
        start_col=0,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        product_stats,
        selected_product_stats,
        combine_product_stats,
        overlay_product_stats,
        ncols=ncols,
        xvar=xvar,
        hue="product",
        title="product Figures",
        sectioned=sectioned,
        cols=None,
        start_col=0,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        type_stats,
        selected_type_stats,
        combine_type_stats,
        overlay_type_stats,
        ncols=ncols,
        xvar=xvar,
        hue="type",
        title="Agent Type Figures",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        agent_stats,
        selected_agent_stats,
        combine_agent_stats,
        overlay_agent_stats,
        ncols=ncols,
        xvar=xvar,
        hue="name",
        title="Agent Instance Figures",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        contract_stats_world,
        selected_contract_stats_world,
        combine_contract_stats_world,
        overlay_contract_stats_world,
        ncols=ncols,
        xvar=xvar,
        hue="world",
        title="Trade Figures (World)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        contract_stats_type,
        selected_contract_stats_type,
        combine_contract_stats_type,
        overlay_contract_stats_type,
        ncols=ncols,
        xvar=xvar,
        hue="agent_type",
        title="Trade Figures (Agent Type)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )

    cols, start_col = add_stats_display(
        contract_stats_buyer_type,
        selected_contract_stats_buyer_type,
        combine_contract_stats_buyer_type,
        overlay_contract_stats_buyer_type,
        ncols=ncols,
        xvar=xvar,
        hue="buyer_type",
        title="Trade Figures (Buyer Type)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        contract_stats_seller_type,
        selected_contract_stats_seller_type,
        combine_contract_stats_seller_type,
        overlay_contract_stats_seller_type,
        ncols=ncols,
        xvar=xvar,
        hue="seller_type",
        cols=cols,
        start_col=start_col,
        title="Trade Figures (Seller Type)",
        sectioned=sectioned,
        dynamic=dynamic,
        ci_level=ci_level,
    )

    cols, start_col = add_stats_display(
        contract_stats_agent,
        selected_contract_stats_agent,
        combine_contract_stats_agent,
        overlay_contract_stats_agent,
        ncols=ncols,
        xvar=xvar,
        hue="agent",
        cols=cols,
        start_col=start_col,
        title="Trade Figures (Agent Instance)",
        sectioned=sectioned,
        dynamic=dynamic,
        ci_level=ci_level,
    )

    cols, start_col = add_stats_display(
        contract_stats_buyer,
        selected_contract_stats_buyer,
        combine_contract_stats_buyer,
        overlay_contract_stats_buyer,
        ncols=ncols,
        xvar=xvar,
        hue="buyer",
        cols=cols,
        start_col=start_col,
        title="Trade Figures (Buyer Instance)",
        sectioned=sectioned,
        dynamic=dynamic,
        ci_level=ci_level,
    )
    cols, start_col = add_stats_display(
        contract_stats_seller,
        selected_contract_stats_seller,
        combine_contract_stats_seller,
        overlay_contract_stats_seller,
        ncols=ncols,
        xvar=xvar,
        hue="seller",
        title="Trade Figures (Seller Instance)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )

    cols, start_col = add_stats_display(
        contract_stats_product,
        selected_contract_stats_product,
        combine_contract_stats_product,
        overlay_contract_stats_product,
        ncols=ncols,
        xvar=xvar,
        hue="product",
        title="Trade Figures (Product)",
        sectioned=sectioned,
        cols=cols,
        start_col=start_col,
        dynamic=dynamic,
        ci_level=ci_level,
    )


def display_others(
    folder,
    selected_worlds,
    selected_products,
    selected_agents,
    selected_types,
    selected_steps,
    selected_times,
    data,
    parent=st.sidebar,
):
    # settings = parent.beta_expander("Settings")
    # ncols = settings.number_input("N. Columns", min_value=1, max_value=6)
    if parent.checkbox("Score Distribution", False):
        score_distribution(selected_worlds, selected_agents, selected_types, data, parent=parent)
    if parent.checkbox("Final Score Factors", False):
        score_factors(selected_worlds, selected_agents, selected_types, data, parent=parent)


if __name__ == "__main__":
    import sys

    from streamlit import cli as stcli

    folder = None
    if len(sys.argv) > 1:
        folder = Path(sys.argv[1])

    if st._is_running_with_streamlit:
        main(folder)
    else:
        sys.argv = ["streamlit", "run"] + sys.argv
        sys.exit(stcli.main())



#!/usr/bin/env python
import itertools
import json
import re
import sys
from pathlib import Path
from typing import Dict, Iterable, Optional

import numpy as np
import pandas as pd

__all__ = ["has_visdata", "has_logs", "main", "VISDATA_FOLDER"]

BASE_IGNORE_SET = {"agent_types", "agent_params", "non_competitors", "non_competitor_params"}
BASE_WORLD_PARAMS_IGNORE_SET = {
    "__dir_name",
    "agent_params",
    "agent_types",
    "name",
    "catalog_prices",
    "profiles",
    "exogenous_contracts",
    "active_lines",
    "input_processes",
    "output_processes",
    "expected_income",
    "expected_income_per_process",
    "expected_income_per_step",
    "expected_n_products",
    "input_quantities",
    "output_quantities",
    "product_prices",
    "process_inputs",
    "process_outputs",
    "profit_basis",
}
ASSIGNED_IGNORE_SET = BASE_IGNORE_SET
ASSIGNED_WORLD_PARAMS_IGNORE_SET = BASE_WORLD_PARAMS_IGNORE_SET
VISDATA_FOLDER = "_visdata"
# tournament files
SCORES_FILE = "scores.csv"
ASSIGNED_CONFIGS_FILE = "assigned_configs.json"  # has n_steps, __dir_name
BASE_CONFIGS_FILE = "base_configs.json"  # has n_steps, __dir_name
# WORLD_STATS_FILE = "stats.csv" # has path and world

TOURNAMENT_REQUIRED = [SCORES_FILE, ASSIGNED_CONFIGS_FILE]

# single world files
CONTRACTS_FILE = ("contracts.csv", "contracts_full_info.csv")
NEGOTIATIONS_FILE = "negotiations.csv"
INFO_FILE = "info.json"
BREACHES_FILE = "breaches.csv"
AGENTS_FILE = "agents.csv"
PARAMS_FILE = "params.csv"
STATS_FILE = "stats.csv"
AGENTS_JSON_FILE = "agents.json"
WORLD_REQUIRED = [STATS_FILE]
MAXWORLDS = float("inf")

PATHMAP = dict()


def has_logs(folder: Path, check_tournament=True, check_world=True) -> bool:
    if check_world:
        for f in WORLD_REQUIRED:
            if not (folder / f).exists():
                return False
        return True
    if check_tournament:
        for f in TOURNAMENT_REQUIRED:
            if not (folder / f).exists():
                return False
        return True
    return True


def adjust_type_names(df):
    if df is None or len(df) == 0:
        return df
    for c in df.columns:
        if not c.endswith("type"):
            continue
        df[c] = df[c].str.split(".").str[0] + "." + df[c].str.split(".").str[-1]
    return df

def adjust_column_names(df):
    if df is None or len(df) == 0:
        return df
    return df.rename(dict(config_id="config", tournament_id="tournament", world_id="world", group_id="group"), axis=1)


def nonzero(f):
    return f.exists() and f.stat().st_size > 0


def get_folders(base_folder, main_file, required, ignore=None):
    return [
        _.parent
        for _ in base_folder.glob(f"**/{main_file}")
        if (not ignore or not re.match(ignore, _.name)) and all(nonzero(f) for f in [_.parent / n for n in required])
    ]


def is_tournament(base_folder):
    return all(nonzero(f) for f in [base_folder / n for n in TOURNAMENT_REQUIRED])


def is_world(base_folder):
    return all(nonzero(f) for f in [base_folder / n for n in WORLD_REQUIRED])


def get_torunaments(base_folder, ignore=None):
    return get_folders(base_folder, main_file=ASSIGNED_CONFIGS_FILE, required=TOURNAMENT_REQUIRED, ignore=ignore)


def get_worlds(base_folder, ignore=None):
    return get_folders(base_folder, main_file=AGENTS_FILE, required=WORLD_REQUIRED, ignore=ignore)


def parse_tournament(
    path,
    t_indx,
    base_indx,
    config_ignore_set=BASE_IGNORE_SET,
    assigned_config_ignore_set=ASSIGNED_IGNORE_SET,
    base_world_ignore_set=BASE_WORLD_PARAMS_IGNORE_SET,
    assigned_world_ignore_set=ASSIGNED_WORLD_PARAMS_IGNORE_SET,
):
    base_configs = json.load(open(path / BASE_CONFIGS_FILE))
    if not base_configs:
        return None, None, None, None
    configs = []
    config_groups = []
    group_map = dict()
    def copy_with_ignore(d, rename_dict, ignore_set, recurse_set):
        c = dict()
        for k, v in d.items():
            if k in ignore_set:
                continue
            if k in recurse_set:
                c = {**c, **copy_with_ignore(v, dict(), ignore_set, recurse_set)}
                continue
            if k in rename_dict.keys():
                c[rename_dict[k]] = v
                continue
            c[k] = v
        return c

    recurse_set = ["world_params", "__exact_params", "info"]
    for aconfig in base_configs:
        conf_group_id = ""
        config_set = []
        for c in aconfig:
            conf_group_id = (conf_group_id + ":" + c["config_id"]) if conf_group_id else c["config_id"]
            d = copy_with_ignore(c, dict(config_id="id"), config_ignore_set.union(base_world_ignore_set), recurse_set)
            config_set.append(d)
        config_groups.append(dict(id=conf_group_id))
        for c in config_set:
            c["group"] = conf_group_id
            configs.append(c)
            group_map[c["id"]] = conf_group_id

    assigned_configs = json.load(open(path / ASSIGNED_CONFIGS_FILE))
    if not assigned_configs:
        return None, None, None, None
    scores = pd.read_csv(path / SCORES_FILE).to_dict("records")  # typing: none
    if not scores:
        return None, None, None, None
    worlds = []
    world_indx = dict()
    world_names = set()
    agents = []
    for i, aconfig in enumerate(assigned_configs):
        if i > MAXWORLDS or not aconfig:
            break
        for c in aconfig:
            world_names.add(c["world_params"]["name"])
            p = c["__dir_name"]
            for k, v in PATHMAP.items():
                p = p.replace(k, v)
            d = dict(
                id=c["world_params"]["name"],
                path=p,
                n_agents=len(c["is_default"]),
                tournament=path.name,
                tournament_indx=t_indx,
            )
            d = {
                **copy_with_ignore(c, dict(), assigned_config_ignore_set.union(assigned_world_ignore_set), recurse_set),
                **d,
            }
            d["name"] = c["world_params"]["name"]
            worlds.append(d)
            world_indx[worlds[-1]["id"]] = worlds[-1]["name"]
            _, wa = get_basic_world_info(Path(p), path.name, worlds[-1]["config_id"], group_map.get(worlds[-1]["config_id"], "none"))
            if not _:
                continue
            agents.append(wa)

    agents = pd.concat(agents)

    return config_groups, configs, worlds, agents


def parse_world(path, tname, wname, nsteps, agents, w_indx, base_indx):
    stats = pd.read_csv(path / STATS_FILE, index_col=0)
    # try:
    #     winfo = json.load(open(path / INFO_FILE))
    # except:
    #     winfo = None
    if (path / AGENTS_JSON_FILE).exists():
        scored_agents = set(agents["name"].to_list())
        ag_dict = json.load(open(path / AGENTS_JSON_FILE))
        for k, v in ag_dict.items():
            if k in ("BUYER", "SELLER"):
                v["final_score"] = float("nan")
            elif k in scored_agents:
                a, b = stats[f"score_{k}"].values[-1], float(agents.loc[agents["name"] == k, "final_score"].values)
                assert a == b
                v["final_score"] = float(agents.loc[agents["name"] == k, "final_score"].values)
            else:
                v["final_score"] = stats[f"score_{k}"].values[-1]
        world_agents = pd.DataFrame.from_records(list(ag_dict.values()))
        world_agents["official_score"] = world_agents["name"].isin(scored_agents)
    else:
        world_agents = agents
        world_agents["official_score"] = True
    contracts = pd.DataFrame(
        data=[],
        columns=[
            "id",
            "seller_name",
            "buyer_name",
            "seller_type",
            "buyer_type",
            "delivery_time",
            "quantity",
            "unit_price",
            "signed_at",
            "nullified_at",
            "concluded_at",
            "signatures",
            "issues",
            "seller",
            "buyer",
            "product_name",
            "n_neg_steps",
            "negotiation_id",
            "executed_at",
            "breaches",
            "erred_at",
            "dropped_at",
        ],
    )
    negotiations = pd.DataFrame(
        data=[],
        columns=[
            "id",
            "seller",
            "buyer",
            "seller_type",
            "buyer_type",
            "delivery_step",
            "delivery_relative_time",
            "quantity",
            "unit_price",
            "step",
            "relative_time",
            "timedout",
            "broken",
            "issues",
            "caller",
            "product",
            "rounds",
            "world",
            "tournament",
            "min_quantity",
            "max_quantity",
            "min_delivery_step",
            "max_delivery_step",
            "min_delivery_relative_time",
            "max_delivery_relative_time",
            "min_unit_price",
            "max_unit_price",
        ],
    )

    offers_list = []
    offers = pd.DataFrame(
        data=[],
        columns=[
            "sender",
            "receiver",
            "sender_type",
            "receiver_type",
            "delivery_step",
            "quantity",
            "unit_price",
            "step",
            "negotiation",
            "relative_time",
            "timedout",
            "broken",
            "caller",
            "product",
            "world",
            "tournament",
            "round",
        ],
    )
    for cname in CONTRACTS_FILE:
        if nonzero(path / cname):
            contracts = pd.read_csv(path / cname, index_col=0)
    if nonzero(path / NEGOTIATIONS_FILE):
        negotiations = pd.read_csv(path / NEGOTIATIONS_FILE, index_col=0)
        negotiations = negotiations.loc[
            :,
            [
                "id",
                "agreement",
                "step",
                "timedout",
                "broken",
                "issues",
                "buyer",
                "seller",
                "caller",
                "product",
                "step",
                "offers",
                "ended_at",
            ],
        ]
        negotiations = negotiations.rename(columns=dict(step="rounds"))
        negotiations = negotiations.rename(columns=dict(ended_at="step"))
        negotiations["offers"] = negotiations.offers.apply(lambda x: eval(x) if isinstance(x, str) else x)
        negotiations["rounds"] = negotiations["rounds"] - 1
        for indx, neg in negotiations.iterrows():
            d = dict(
                sender=None,
                receiver=None,
                delivery_step=None,
                quantity=None,
                unit_price=None,
                round=None,
                index=None,
                step=neg["step"],
                timedout=neg["timedout"],
                broken=neg["broken"],
                caller=neg["caller"],
                product=neg["product"],
                world=wname,
                tournament=tname,
            )
            if not neg["offers"]:
                continue
            offers = eval(neg["offers"]) if isinstance(neg["offers"], str) else neg["offers"]
            current_offers = []
            for k, vs in offers.items():
                for r, v in enumerate(vs):
                    dd = {**d}
                    dd["sender"] = k
                    dd["negotiation"] = neg["id"]
                    dd["receiver"] = neg["seller"] if k == neg["buyer"] else neg["buyer"]
                    dd["round"] = r
                    if v:
                        dd["quantity"] = v[0]
                        dd["delivery_step"] = v[1]
                        dd["unit_price"] = v[2]
                    else:
                        dd["quantity"] = None
                        dd["delivery_step"] = None
                        dd["unit_price"] = None

                    current_offers.append(dd)
            for indx, v in enumerate(current_offers):
                v["index"] = indx
            offers_list += current_offers

        atmap = dict(zip(world_agents["name"].to_list(), world_agents["type"].to_list()))
        # atmap = dict(zip(all_agents["name"].to_list(), all_agents["type"].to_list()))
        # negotiations.agreement[negotiations.agreement.isna(), "agreement"] = None
        for c in ["quantity", "delivery_step", "unit_price"]:
            negotiations[c] = None

        def lst(x):
            return (
                [-1, -1, -1]
                if not x or (isinstance(x, str) and x.lower() == "none")
                else list(eval(x))
                if isinstance(x, str)
                else list(x)
            )

        negotiations.agreement = negotiations.agreement.apply(lst)
        agreements = pd.DataFrame(negotiations.agreement.tolist(), index=negotiations.index)
        agreements.columns = ["quantity", "delivery_step", "unit_price"]
        for c in ["quantity", "delivery_step", "unit_price"]:
            negotiations[c] = agreements[c]
            negotiations.loc[negotiations.loc[:, c] < 0, c] = float("nan")

        def do_map(x):
            return atmap[x]

        negotiations["buyer_type"] = negotiations.buyer.apply(do_map)
        negotiations["seller_type"] = negotiations.seller.apply(do_map)
        negotiations["world"] = wname
        negotiations["tournament"] = tname
        negotiations = negotiations.drop("agreement", axis=1)

        def get_issue_ranges(x):
            if isinstance(x, str):
                x = eval(x)
            if not isinstance(x, Iterable):
                return x
            y = []
            for s in x:
                mn, mx = eval(s.split(":")[-1].strip())
                y += [mn, mx]
            return y

        negotiations = negotiations.reset_index(None)
        issues = negotiations.issues.apply(get_issue_ranges)
        issues = pd.DataFrame(
            issues.to_list(),
            columns=[
                "min_quantity",
                "max_quantity",
                "min_delivery_step",
                "max_delivery_step",
                "min_unit_price",
                "max_unit_price",
            ],
        )
        negotiations = pd.concat((negotiations, issues), axis=1)
        if len(negotiations):
            negotiations.drop("issues", axis=1, inplace=True)
            for c in negotiations.columns:
                if not c.endswith("step") and not c.endswith("steps"):
                    continue
                negotiations.loc[negotiations[c] < 0, c] = float("nan")
                negotiations[c.replace("step", "relative_time")] = negotiations[c] / nsteps if nsteps else 0.0
        offers = pd.DataFrame.from_records(offers_list)
        if len(offers):
            offers["receiver_type"] = offers.receiver.apply(do_map)
            offers["sender_type"] = offers.sender.apply(do_map)
            for c in offers.columns:
                if not c.endswith("step") and not c.endswith("steps"):
                    continue
                offers.loc[offers[c] < 0, c] = float("nan")
                offers[c.replace("step", "relative_time")] = offers[c] / nsteps if nsteps else 0.0

    contracts = contracts.loc[
        :,
        [
            "id",
            "seller_type",
            "buyer_type",
            "delivery_time",
            "quantity",
            "unit_price",
            "signed_at",
            "nullified_at",
            "concluded_at",
            # "issues",
            "seller",
            "buyer",
            "product_name",
            "n_neg_steps",
            "negotiation_id",
            "executed_at",
            "breaches",
            "erred_at",
            "dropped_at",
        ],
    ]
    contracts = contracts.rename(
        columns=dict(
            delivery_time="delivery_step",
            negotiation_id="negotiation",
            product_name="product",
        )
    )
    contracts["product"] = contracts["product"].str[1:].astype(int)
    contracts.columns = [_.replace("_at", "_step") if _.endswith("_at") else _ for _ in contracts.columns]
    # contracts["step"] = contracts["concluded_step"]
    contracts["breached_step"] = contracts["executed_step"]
    contracts["breaches"] = contracts.breaches.astype(str)
    contracts.loc[(contracts.signed_step >= 0) & (contracts.executed_step < 0), "breached_step"] = -1
    contracts.loc[
        (contracts.signed_step >= 0) & (contracts.breaches.str.len() > 0),
        "breached_step",
    ] = -1
    contracts["world"] = wname
    contracts["tournament"] = tname
    # step_cols = (
    #     "",
    #     "concluded_",
    #     "signed_",
    #     "executed_",
    #     "erred_",
    #     "dropped_",
    #     "nullified_",
    #     "breached_",
    # )
    for c in contracts.columns:
        if not c.endswith("step") and not c.endswith("steps"):
            continue
        contracts.loc[contracts[c] < 0, c] = float("nan")
        contracts[c.replace("step", "relative_time")] = contracts[c] / nsteps if nsteps else 0.0

    contract_stats, negotiation_stats = [], []
    non_step_cols = [
        "unit_price",
        "quantity",
        "product",
        "buyer",
        "seller",
        "buyer_type",
        "seller_type",
    ]

    def calc_qp(x):
        c = len(x)
        s = (x["quantity"] * x["unit_price"]).sum()
        q = x["quantity"].sum()
        if q:
            return q, c, s / q
        return 0, 0, 0.0

    for step in range(nsteps):
        s = None
        for c in contracts.columns:
            if not c.endswith("step"):
                continue
            base = c.replace("step", "").replace("_", "")
            qp = contracts.loc[contracts[c] == step, non_step_cols].groupby(non_step_cols[2:]).apply(calc_qp)
            if not isinstance(qp, pd.DataFrame):
                qp = pd.DataFrame(data=qp, index=qp.index)
            if len(qp) < 1:
                continue
            # breakpoint()
            qp.columns = ["qp"]
            # breakpoint()
            qp = pd.DataFrame(qp["qp"].tolist(), index=qp.index)
            qp = qp.rename(columns={0: base + "_quantity", 1: base + "_count", 2: base + "_unit_price"})
            # qp = qp.reset_index()
            qp["step"] = step
            qp.set_index(["step"], append=True, inplace=True)
            qp = qp.melt(
                value_vars=[base + "_quantity", base + "_count", base + "_unit_price"],
                ignore_index=False,
            )
            if s is None:
                s = qp
            else:
                s = pd.concat((s, qp), ignore_index=False, axis=0)
        if s is not None:
            contract_stats.append(s.reset_index())
    # contract_stats = pd.DataFrame.from_records(contract_stats)
    contract_stats = pd.concat(contract_stats, ignore_index=True)
    contract_stats = contract_stats.pivot(
        index=[_ for _ in contract_stats.columns if _ not in ("variable", "value")],
        columns="variable",
        values="value",
    ).reset_index()
    contract_stats["relative_time"] = (contract_stats["step"] / nsteps) if nsteps else 0.0
    contract_stats = contract_stats.fillna(0)
    contract_stats["world"] = wname
    contract_stats["tournament"] = tname
    for col in [_ for _ in contract_stats.columns if _.endswith("unit_price")]:
        field = col.split("_")[0]
        if f"{field}_quantity" in contract_stats.columns:
            contract_stats[f"{field}_total_price"] = (
                contract_stats[f"{field}_quantity"] * contract_stats[f"{field}_unit_price"]
            )

    for c in contract_stats.columns:
        if c.endswith("quantity"):
            contract_stats[c] = contract_stats[c].astype(int)
        if c.endswith("count"):
            contract_stats[c] = contract_stats[c].astype(int)
        if c.endswith("unit_price"):
            contract_stats[c] = contract_stats[c].astype(float)

    # contract_stats.drop_duplicates(inplace=True, subset=["buyer", "seller", "step"])

    if nonzero(path / BREACHES_FILE):
        breaches = pd.read_csv(path / BREACHES_FILE, index_col=0)
    else:
        breaches = pd.DataFrame(
            data=[],
            columns=[
                "contract",
                "contract_id",
                "type",
                "level",
                "id",
                "perpetrator",
                "perpetrator_type",
                "victims",
                "step",
                "resolved",
            ],
        )
    breaches = breaches.loc[
        :,
        [
            "step",
            "contract_id",
            "type",
            "level",
            "id",
            "perpetrator",
            "perpetrator_type",
            "victims",
            "resolved",
        ],
    ]
    breaches = breaches.rename(columns=dict(contract_id="contract"))
    breaches["victim"] = breaches["victims"].apply(lambda x: x[0] if not isinstance(x, str) else eval(x)[0])
    # contracts["relative_time"] = breaches["step"] / nsteps
    breaches.drop(["victims"], axis=1)
    agent_names = world_agents["name"].unique()
    # inventory_{}_input", output
    product_stat_names = ["trading_price", "sold_quantity", "unit_price"]
    products = set([_.split("_")[-1] for _ in stats.columns if any(_.startswith(p) for p in product_stat_names)])
    agent_stat_names = [
        "balance",
        "productivity",
        "bankrupt",
        "score",
        "assets",
        "inventory-input",
        "inventory-output",
        "spot_market_loss",
        "spot_market_quantity",
    ]
    agents_info, product_info = [], []
    world_stat_names = list(
        str(_)
        for _ in stats.columns
        if not any(_.startswith(n.split("-")[0]) for n in (agent_stat_names + product_stat_names))
    )
    for aname in agent_names:
        colnames = []
        ainfo = world_agents.loc[world_agents["name"] == aname, ["type", "final_score"]].to_dict("records")[0]
        for n in agent_stat_names:
            ns = n.split("-")
            if len(ns) > 1:
                col_name = f"{ns[0]}_{aname}_{ns[-1]}"
            else:
                col_name = f"{n}_{aname}"
            if col_name not in stats.columns:
                continue
            colnames.append(col_name)
        x = stats.loc[:, colnames].reset_index().rename(columns=dict(index="step"))
        x.columns = [_ if aname not in _ else _.replace(f"_{aname}", "").replace("-", "_") for _ in x.columns]
        if len(x):
            x["relative_time"] = x["step"] / nsteps
        else:
            x["relative_time"] = 0.0
        x["name"] = aname
        x["world"] = wname
        x["tournament"] = tname
        for k, v in ainfo.items():
            x[k] = v
        agents_info.append(x)

    for p in products:
        colnames = []
        for n in product_stat_names:
            col_name = f"{n}_{p}"
            colnames.append(col_name)
        x = stats.loc[:, colnames].reset_index().rename(columns=dict(index="step"))
        x.columns = ["_".join(_.split("_")[:-1]) if _.endswith(f"_{p}") else _ for _ in x.columns]
        if len(x):
            x["relative_time"] = x["step"] / nsteps
        else:
            x["relative_time"] = 0.0
        x["product"] = p
        x["world"] = wname
        x["tournament"] = tname
        product_info.append(x)

    world_info = stats.loc[:, world_stat_names].reset_index().rename(columns=dict(index="step"))
    if len(world_info):
        world_info["relative_time"] = world_info["step"] / nsteps
    else:
        world_info["relative_time"] = 0.0
    world_info["world"] = wname
    world_info["tournament"] = tname

    return (
        pd.concat(agents_info, ignore_index=True),
        pd.concat(product_info, ignore_index=True),
        world_info,
        contracts,
        contract_stats,
        negotiations,
        offers,
        negotiation_stats,
        breaches,
    )


def get_basic_world_info(path, tname, gname, cname):
    try:
        stats = pd.read_csv(path / STATS_FILE, index_col=0).to_dict("list")
        adata = json.load(open(path / AGENTS_JSON_FILE))
        winfo = json.load(open(path / INFO_FILE))
    except:
        print(f"FAILED {path.name}", flush=True)
        return [], None
    worlds = [dict(name=path.name, tournament=tname, config=cname, group=gname, tournament_indx=0, path=path, n_steps=winfo["n_steps"])]
    agents = []
    definfo = winfo.get("is_default", None)
    agent_key = None
    for k in ("agent_initial_balances", "agent_profiles", "agent_inputs", "agent_outputs", "agent_processes"):
        if k in winfo.keys():
            agent_key = k
            break
    if definfo:
        is_default = dict(zip(winfo[agent_key].keys(), definfo))
    else:
        is_default = dict(zip(winfo[agent_key].keys(), itertools.repeat(False)))
    for i, (aname, info) in enumerate(adata.items()):
        if f"score_{aname}" not in stats.keys():
            continue
        score = stats[f"score_{aname}"][-1]
        aginfo = winfo["agent_profiles"][aname]
        aginfo["initial_balance"] = winfo.get("agent_initial_balances", dict()).get(aname, float("nan"))
        aginfo["is_default"] = is_default.get(aname, True)
        if "costs" in aginfo.keys():
            aginfo["cost"] = float(np.asarray(aginfo["costs"]).min())
            del aginfo["costs"]
        dd = dict(id=i, name=aname, world=worlds[0]["name"], tournament=tname, group=gname, config=cname, final_score=score, type=info["type"])
        dd = {**dd, **aginfo}
        agents.append(dd)
    return worlds, pd.DataFrame.from_records(agents)


def get_data(base_folder, ignore: Optional[str] = None):
    base_folder = Path(base_folder)
    tournaments, worlds, agents, agent_stats, product_stats, world_stats = (
        [],
        [],
        [],
        [],
        [],
        [],
    )
    groups, configs = [], []
    contracts, contract_stats, breaches = [], [], []
    negotiations, offers, neg_stats = [], [], []
    if is_tournament(base_folder):
        paths = get_torunaments(base_folder, ignore)
    elif is_world(base_folder):
        paths = [None]
    else:
        paths = get_torunaments(base_folder, ignore)
        paths += get_worlds(base_folder, ignore)
        # raise ValueError(f"Folder {str(base_folder)} contains neither tournament nor world logs")
    none_tournament = dict(id="none", path=base_folder.parent, name="none")
    none_group = dict(id="none")
    none_config = dict(id="none", name="none")
    none_added = False
    for i, t in enumerate(paths):
        indx = i + 1
        base_indx = (i + 1) * 1_000_000
        if t is not None:
            if is_tournament(t):
                print(f"Tournament {t.name} [{i} of {len(paths)}]", flush=True)
                tournaments.append(dict(id=indx, path=t, name=t.name))
                g, con, w, a = parse_tournament(t, indx, base_indx)
                tname = t.name
            else:
                print(f"World {t.name} [{i} of {len(paths)}]", flush=True)
                if not none_added:
                    tournaments.append(none_tournament)
                    none_added = True
                w, a = get_basic_world_info(base_folder, "none", "none", "none")
                g = [{k: v for k, v in none_group.items()}]
                con = [{k: v for k, v in none_config.items()}]
                tname = "none"
        else:
            print(f"No tournament found ... reading world info")
            tname = "none"
            tournaments.append(dict(id=tname, path=base_folder.parent, name=tname))
            g = [{k: v for k, v in none_group.items()}]
            con = [{k: v for k, v in none_config.items()}]
            w, a = get_basic_world_info(base_folder, tname, "none", "none")
        for gg in g:
            gg["tournament"] = tname
        for cc in con:
            cc["tournament"] = tname
        groups += g
        configs += con
        if not w:
            continue
        for j, world in enumerate(w):
            wagents = a.loc[a.world == world["name"]]
            try:
                ag, pr, wo, co, cs, ng, of, ns, br = parse_world(
                    Path(world["path"]),
                    tname,
                    world["name"],
                    world["n_steps"],
                    wagents,
                    base_indx + j + 1,
                    base_indx + j + 1,
                )
            except:
                print(f"\tParse Error {world['name']}", flush=True)
                continue
            print(f"\tWorld {world['name']} [{j} of {len(w)}]", flush=True)
            if ag is not None and len(ag):
                agent_stats.append(ag)
            if pr is not None and len(pr):
                product_stats.append(pr)
            if wo is not None and len(wo):
                world_stats.append(wo)
            if co is not None and len(co):
                contracts.append(co)
            if cs is not None and len(cs):
                contract_stats.append(cs)
            if ng is not None and len(ng):
                negotiations.append(ng)
            if of is not None and len(of):
                offers.append(of)
            if ns is not None and len(ns):
                neg_stats.append(ns)
            if br is not None and len(br):
                breaches.append(br)
        if w is not None and len(w):
            worlds.append(pd.DataFrame.from_records(w))
        if a is not None and len(a):
            agents.append(a)

    tournaments = pd.DataFrame.from_records(tournaments)
    groups = pd.DataFrame.from_records(groups)
    configs = pd.DataFrame.from_records(configs)
    if worlds is not None and len(worlds):
        worlds = pd.concat(worlds, ignore_index=True)
    if agents is not None and len(agents):
        agents = pd.concat(agents, ignore_index=True)
    if agent_stats is not None and len(agent_stats):
        agent_stats = pd.concat(agent_stats, ignore_index=True)
    if product_stats is not None and len(product_stats):
        product_stats = pd.concat(product_stats, ignore_index=True)
    if world_stats is not None and len(world_stats):
        world_stats = pd.concat(world_stats, ignore_index=True)
    if contracts is not None and len(contracts):
        contracts = pd.concat(contracts, ignore_index=True)
    if contract_stats is not None and len(contract_stats):
        contract_stats = pd.concat(contract_stats, ignore_index=True)
    if negotiations is not None and len(negotiations):
        negotiations = pd.concat(negotiations, ignore_index=True)
    if offers is not None and len(offers):
        offers = pd.concat(offers, ignore_index=True)
    if neg_stats is not None and len(neg_stats):
        neg_stats = pd.concat(neg_stats, ignore_index=True)
    if breaches is not None and len(breaches):
        breaches = pd.concat(breaches, ignore_index=True)
    return (
        tournaments,
        groups,
        configs,
        worlds,
        agents,
        agent_stats,
        product_stats,
        world_stats,
        contracts,
        contract_stats,
        negotiations,
        offers,
        neg_stats,
        breaches,
    )


def map_paths(folder: Path, m: Dict[str, str]):
    """Maps paths inside all files within the given folder"""

    for fname in folder.glob("**/*"):
        if fname.is_dir():
            continue
        name = fname.name
        if not name.endswith("json") and not name.endswith("csv"):
            continue
        with open(fname, "r") as infile:
            s = infile.read()
        for k, v in m.items():
            s = s.replace(k, v)
        with open(fname, "w") as outfile:
            outfile.write(s)


def main(folder: Path, max_worlds: Optional[int], ignore: Optional[str] = None, pathmap: Optional[str] = None):
    folder = Path(folder)
    if max_worlds is None:
        max_worlds = float("inf")
    dst_folder = folder / VISDATA_FOLDER
    if dst_folder.exists():
        raise ValueError(
            f"Destiantion folder {dst_folder} exists. Delete it if you want to recompile visualization data"
        )
    folder = Path(folder)

    if pathmap:
        m = pathmap.split(":")
        map_paths(folder, dict(zip(m[0::2], m[1::2])))

    (
        tournaments,
        groups,
        configs,
        worlds,
        agents,
        agent_stats,
        product_stats,
        world_stats,
        contracts,
        contract_stats,
        negotiations,
        offers,
        neg_stats,
        breaches,
    ) = get_data(folder, ignore=ignore)
    dst_folder = folder / VISDATA_FOLDER
    dst_folder.mkdir(parents=True, exist_ok=True)
    for df, name in zip(
        (
            tournaments,
            groups,
            configs,
            worlds,
            agents,
            agent_stats,
            product_stats,
            world_stats,
            contracts,
            contract_stats,
            negotiations,
            offers,
            neg_stats,
            breaches,
        ),
        (
            "tournaments",
            "groups",
            "configs",
            "worlds",
            "agents",
            "agent_stats",
            "product_stats",
            "world_stats",
            "contracts",
            "contract_stats",
            "negotiations",
            "offers",
            "neg_stats",
            "breaches",
        ),
    ):
        if df is None or len(df) == 0:
            print(f"\tDid not find {name}")
            continue
        df = adjust_type_names(df)
        df = adjust_column_names(df)
        df.to_csv(dst_folder / f"{name}.csv", index=False)


def has_visdata(folder: Path):
    folder = Path(folder)
    files = (
        "tournaments",
        "worlds",
        "agents",
        "agent_stats",
        "product_stats",
        "world_stats",
    )
    if folder.name != VISDATA_FOLDER:
        folder /= VISDATA_FOLDER
    if not folder.exists():
        return False
    for file in files:
        if not nonzero(folder / f"{file}.csv"):
            return False
    return True


if __name__ == "__main__":
    import sys

    for arg in sys.argv[1:]:
        if arg == "--map":
            PATHMAP = {"/export/home": "/Users"}
        elif arg.startswith("n="):
            MAXWORLDS = int(arg.split("=")[-1])
        else:
            path = arg
    main(Path(path), MAXWORLDS)



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['scml_vis', 'scml_vis.vendor', 'scml_vis.vendor.quick']

package_data = \
{'': ['*']}

install_requires = \
['PyQt5>=5.15.4,<6.0.0',
 'click-config-file>=0.6.0,<0.7.0',
 'click>=7.1.2,<8.0.0',
 'csvs-to-sqlite>=1.2,<2.0',
 'datasette-vega>=0.6.2,<0.7.0',
 'numpy>=1.20.2,<2.0.0',
 'pandas>=1.2.3,<2.0.0',
 'plotly>=4.14.3,<5.0.0',
 'seaborn>=0.11.1,<0.12.0',
 'streamlit>=0.80.0,<0.81.0',
 'watchdog>=2.0.2,<3.0.0']

entry_points = \
{'console_scripts': ['scml-vis = scml_vis.cli:main',
                     'scmlv = scml_vis.cli:main',
                     'scmlvis = scml_vis.cli:main']}

setup_kwargs = {
    'name': 'scml-vis',
    'version': '0.2.7',
    'description': 'A simple visualiser for SCML worlds and tournaments',
    'long_description': '# scml-vis\n\n[![ci](https://github.com/yasserfarouk/scml-vis/actions/workflows/main.yml/badge.svg)](https://github.com/yasserfarouk/scml-vis/actions/workflows/main.yml)\n[![documentation](https://img.shields.io/badge/docs-mkdocs%20material-blue.svg?style=flat)](https://scml-vis.github.io/scml-vis/)\n[![pypi version](https://img.shields.io/pypi/v/scml-vis.svg)](https://pypi.org/project/scml-vis/)\n[![gitter](https://badges.gitter.im/join%20chat.svg)](https://gitter.im/scml-vis/community)\n\nA simple visualiser for SCML worlds and tournaments.\nYou can watch [a demo on YouTube](https://youtu.be/BCDjnnSmIsk)\n[![Alt text](https://img.youtube.com/vi/BCDjnnSmIsk/0.jpg)](https://youtu.be/BCDjnnSmIsk)\n<!-- <iframe width="560" height="315" src="https://www.youtube.com/embed/BCDjnnSmIsk" title="YouTube video player" frameborder="0" allow="accelerometer; autoplay; clipboard-write; encrypted-media; gyroscope; picture-in-picture" allowfullscreen></iframe> -->\n<!--  -->\n## Screenshots\n![Screen Shot 1](docs/shot1.png)\n![Screen Shot 2](docs/shot2.png)\n\n## Main Features\n\n- Displays any world/tournament run using the [SCML package](https://www.github.com/yasserfarouk/scml)\n- Allows filtering using worlds, agent types, and agent instances\n- Shows world statistics, agent type and instance statistics and contract\n  statistics as functions of simulation step/time\n\n\n## Requirements\n\n- scml-vis requires Python 3.8 or above.\n- scml-vis can visualize worlds created by [scml](https://github.com/yasserfarouk/scml) 0.4.2 or later (simulations created using older versions of scml can be visualized in most cases but are not officially supported).\n\n## Installation\n\nWith `pip`:\n```bash\npython3 -m pip install scml-vis\n```\n\nWith [`pipx`](https://github.com/pipxproject/pipx):\n```bash\npython3 -m pip install --user pipx\n\npipx install scml-vis\n```\n\n## Usage\n\nThe visualizer can be run using any of the following commands:\n```bash\nscmlv\nscmlvis\nscml-vis\n```\nHereafter we will use the shorter version.\n\n- To visualize any of the recently run worlds and tournaments just run:\n\t```bash\n\tscmlv show\n\t```\n\tThis will open your browser and allow you to choose a world or a tournament to\n\tdisplay.\n\t![show without parameters](docs/show.png)\n\n\t- If this is the first time you visualize logs in this folder, you will be asked\n\t  to compile visualization data\n\t\t![compile visualization data](docs/compile.png)\n\t- If visualization data is available (or your pressed the `compile visualization data` button), you can now start visualizing the logs\n\t\t![compile visualization data](docs/aftercompile.png)\n- To visualize the logs in a specific folder, you can directly pass the folder as in:\n  ```bash\n\tscmlv show -f path-to-your-folder\n\t```\n- It is also possible to just compile visualization data without running the visualizer using:\n  ```bash\n\tscmlv compile path-to-your-folder\n\t```\n- The visualizer creates a database that it uses to create all the figures you see. You can directly explore this database using:\n  ```bash\n\tscmlv explore path-to-your-folder\n\t```\n\tThis will open a [datasette](https://docs.datasette.io/en/stable/getting_started.html) page allowing you to explore this database\n\t![datasette](docs/datasette.png)\n\tThis dataset will contain 8 tables describing everything that was logged in the world or tournament. \n\t![datasette](docs/datasettelarge.png)\n\tPlease consult [datasette documentation](https://docs.datasette.io/en/stable/getting_started.html) for all the ways you can interact with this dataset.\n\n## Available visualizations\n\nTo visualize your logs, you need to follow three steps:\n\n1. Filter the dataset using the `Data Selection` section of the sidebar.\n2. Choose the **family** of figures you would like to show from the `Figure Selection` dropdown in the sidebar. Currently we provide `Time-series`, `Tables` and `others` that are always available as well as `Networks`  that are only available when you visualize a single world or filter the tournament to focus on a single world.\n3. Choose the specific graph you want to see from the family selected in the previous step.\n\nYou can watch [a demo of this process here](https://youtu.be/BCDjnnSmIsk)\n\n## TODO List (Good Ideas for PRs)\n\n- ~~Show negotiation logs (i.e. negotiation results)~~\n- ~~Display all contracts (i.e. in a table) based on selection criteria~~\n- ~~Zoom on negotiation details (i.e. exchanged offers)~~\n- ~~Add dynamic figures using plotly/altair~~\n- ~~Add networkx like graphs of contracts / negotiations / offers~~\n- ~~Allow starting the app without specifying a folder.~~\n- Add saving and loading of the visualizer\'s state (i.e. what is visible).\n- Add new figure types that do not have time/step in the x-axis.\n- Correcting the placement of weights on edges in network views.\n- Adding a graph showing negotiation history in the ufun-space of negotiators (will require a change in the scml package).\n- Resolving the strange behavior of CI bands in plotly in some cases.\n',
    'author': 'Yasser Mohammad',
    'author_email': 'yasserfarouk@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/scml-vis/scml-vis',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)
